@@ -1,9 +1,9 @@
 import sys
 from pathlib import Path
 
-__version__ = '1.9.3'
+from .config import *  # noqa: F403
 
-IS_FLATPAK = Path('/app/share/run-as-flatpak').exists()
+__version__ = '1.9.4'
 
 portable_path = Path(sys.executable).parent / 'is_portable'
 IS_PORTABLE = portable_path.exists()
