@@ -151,6 +151,10 @@ def is_flatpak() -> bool:
     return gajim.IS_FLATPAK
 
 
+def is_flatpak_nightly() -> bool:
+    return gajim.IS_FLATPAK_NIGHTLY
+
+
 def is_portable() -> bool:
     return gajim.IS_PORTABLE
 
@@ -159,6 +163,13 @@ def is_ms_store() -> bool:
     return gajim.IS_MS_STORE
 
 
+def get_default_app_id() -> str:
+    app_id = 'org.gajim.Gajim'
+    if is_flatpak_nightly():
+        return f'{app_id}.Devel'
+    return app_id
+
+
 def is_display(display: Display) -> bool:
     # XWayland reports as Display X11, so try with env var
     is_wayland = os.environ.get('XDG_SESSION_TYPE') == 'wayland'
