@@ -12,22 +12,11 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
 import logging
 import socket
@@ -48,6 +37,9 @@ from gajim.common.file_props import FileProp
 from gajim.common.file_props import FilesProp
 from gajim.common.modules.base import BaseModule
 from gajim.common.socks5 import Socks5SenderClient
+from gajim.common.util.jid import get_full_jid_from_iq
+from gajim.common.util.jid import get_jid_from_iq
+from gajim.common.util.jid import parse_jid
 
 log = logging.getLogger('gajim.c.m.bytestream')
 
@@ -130,12 +122,12 @@ class Bytestream(BaseModule):
         return file_props.receiver.jid + '/' + file_props.receiver.resource
 
     @staticmethod
-    def _ft_get_from(iq_obj: Iq) -> Optional[str]:
-        return helpers.get_full_jid_from_iq(iq_obj)
+    def _ft_get_from(iq_obj: Iq) -> str | None:
+        return get_full_jid_from_iq(iq_obj)
 
     @staticmethod
     def _ft_get_streamhost_jid_attr(streamhost: nbxmpp.Node) -> str:
-        return helpers.parse_jid(streamhost.getAttr('jid'))
+        return parse_jid(streamhost.getAttr('jid'))
 
     def send_file_approval(self, file_props: FileProp) -> None:
         '''
@@ -183,7 +175,7 @@ class Bytestream(BaseModule):
             session.cancel_session()
 
     def send_success_connect_reply(self,
-                                   streamhost: Optional[dict[str, Any]]
+                                   streamhost: dict[str, Any] | None
                                    ) -> None:
         '''
         Send reply to the initiator of FT that we made a connection
@@ -230,7 +222,7 @@ class Bytestream(BaseModule):
         for file_props in FilesProp.getAllFileProp():
             self.remove_transfer(file_props)
 
-    def remove_transfer(self, file_props: Optional[FileProp]) -> None:
+    def remove_transfer(self, file_props: FileProp | None) -> None:
         if file_props is None:
             return
         self.disconnect_transfer(file_props)
@@ -519,7 +511,7 @@ class Bytestream(BaseModule):
                        sid: str,
                        error: str,
                        error_type: str,
-                       msg: Optional[str] = None
+                       msg: str | None = None
                        ) -> None:
         '''
         Called when there is an error establishing BS connection, or when
@@ -573,10 +565,10 @@ class Bytestream(BaseModule):
                              _properties: IqProperties
                              ) -> None:
         id_ = iq_obj.getAttr('id')
-        frm = helpers.get_full_jid_from_iq(iq_obj)
+        frm = get_full_jid_from_iq(iq_obj)
         query = iq_obj.getTag('query')
         app.proxy65_manager.error_cb(frm, query)
-        jid = helpers.get_jid_from_iq(iq_obj)
+        jid = get_jid_from_iq(iq_obj)
         id_ = id_[3:]
         file_props = FilesProp.getFilePropBySid(id_)
         if not file_props:
