@@ -10,7 +10,6 @@ from typing import cast
 
 import mimetypes
 import os
-import tempfile
 from collections import defaultdict
 from collections.abc import Callable
 from pathlib import Path
@@ -31,18 +30,19 @@ from nbxmpp.task import Task
 from nbxmpp.util import convert_tls_error_flags
 
 from gajim.common import app
+from gajim.common import configpaths
 from gajim.common import types
 from gajim.common.const import FTState
 from gajim.common.events import HTTPUploadError
 from gajim.common.events import HTTPUploadStarted
 from gajim.common.exceptions import FileError
 from gajim.common.filetransfer import FileTransfer
-from gajim.common.helpers import get_random_string
-from gajim.common.helpers import get_tls_error_phrases
 from gajim.common.i18n import _
 from gajim.common.modules.base import BaseModule
 from gajim.common.structs import OutgoingMessage
 from gajim.common.util.http import create_http_request
+from gajim.common.util.text import get_random_string
+from gajim.common.util.user_strings import get_tls_error_phrases
 
 
 class HTTPUpload(BaseModule):
@@ -420,8 +420,8 @@ class HTTPFileTransfer(FileTransfer):
 
     @staticmethod
     def _get_temp_path() -> Path:
-        tempdir = tempfile.gettempdir()
-        return Path(tempdir) / get_random_string(16)
+        tempdir = configpaths.get('TMP')
+        return tempdir / get_random_string(16)
 
     def set_error(self, domain: str, text: str = '') -> None:
         if not text:
