@@ -22,11 +22,9 @@ from nbxmpp.structs import MucSubject
 from gajim.common import app
 from gajim.common import events
 from gajim.common import ged
-from gajim.common import helpers
 from gajim.common import types
 from gajim.common.const import XmppUriQuery
 from gajim.common.ged import EventHelper
-from gajim.common.helpers import get_moderation_text
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
@@ -34,8 +32,13 @@ from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.modules.httpupload import HTTPFileTransfer
 from gajim.common.storage.archive.const import ChatDirection
 from gajim.common.storage.archive.models import Message
+from gajim.common.util.status import get_uf_show
+from gajim.common.util.user_strings import get_moderation_text
+from gajim.common.util.user_strings import get_uf_affiliation
+from gajim.common.util.user_strings import get_uf_role
 
 from gajim.gtk.builder import get_builder
+from gajim.gtk.conversation.chatstate_indicator import ChatstateIndicator
 from gajim.gtk.conversation.jump_to_end_button import JumpToEndButton
 from gajim.gtk.conversation.message_selection import MessageSelection
 from gajim.gtk.conversation.rows.widgets import MessageRowActions
@@ -77,6 +80,9 @@ class ChatControl(EventHelper):
         self._message_selection.connect('cancel', self._on_cancel_selection)
         self._ui.conv_view_overlay.add_overlay(self._message_selection)
 
+        self._chatstate_indicator = ChatstateIndicator()
+        self._ui.conv_view_overlay.add_overlay(self._chatstate_indicator)
+
         self._jump_to_end_button = JumpToEndButton()
         self._jump_to_end_button.connect('clicked', self._on_jump_to_end)
         self._ui.conv_view_overlay.add_overlay(self._jump_to_end_button)
@@ -186,20 +192,12 @@ class ChatControl(EventHelper):
     ) -> None:
         row = self._scrolled_view.get_row_by_pk(pk)
         if row is None:
-            m = app.storage.archive.get_message_with_pk(pk)
-            if m is None:
-                log.info('Message with pk %s was not found in DB', pk)
-                return
-
             # Clear view and reload conversation around timestamp
             self._scrolled_view.reset()
             self._scrolled_view.block_signals(True)
-            messages: list[Message] = []
-            messages.append(m)
-            messages.extend(app.storage.archive.get_conversation_before_after(
-                self.contact.account, self.contact.jid, True, timestamp, 50))
-            messages.extend(app.storage.archive.get_conversation_before_after(
-                self.contact.account, self.contact.jid, False, timestamp, 50))
+            messages = app.storage.archive.get_conversation_around_timestamp(
+                self.contact.account, self.contact.jid, timestamp
+            )
             self._add_messages(messages)
             self._scrolled_view.set_history_complete(False, False)
 
@@ -224,6 +222,7 @@ class ChatControl(EventHelper):
 
         self._client = app.get_client(contact.account)
 
+        self._chatstate_indicator.switch_contact(contact)
         self._jump_to_end_button.switch_contact(contact)
         self._message_row_actions.switch_contact(contact)
         self._scrolled_view.switch_contact(contact)
@@ -248,6 +247,7 @@ class ChatControl(EventHelper):
                 'user-left': self._on_user_left,
                 'user-affiliation-changed': self._on_user_affiliation_changed,
                 'user-role-changed': self._on_user_role_changed,
+                'user-hats-changed': self._on_user_hats_changed,
                 'user-status-show-changed': self._on_user_status_show_changed,
                 'user-nickname-changed': self._on_user_nickname_changed,
                 'room-kicked': self._on_room_kicked,
@@ -611,6 +611,9 @@ class ChatControl(EventHelper):
             elif isinstance(row, events.MUCUserRoleChanged):
                 self._process_muc_user_role_changed(row)
 
+            elif isinstance(row, events.MUCUserHatsChanged):
+                self._process_muc_user_hats_changed(row)
+
             elif isinstance(row, events.MUCUserStatusShowChanged):
                 self._process_muc_user_status_show_changed(row)
 
@@ -746,7 +749,7 @@ class ChatControl(EventHelper):
             self,
             event: events.MUCUserAffiliationChanged) -> None:
 
-        affiliation = helpers.get_uf_affiliation(event.affiliation)
+        affiliation = get_uf_affiliation(event.affiliation)
 
         reason = event.reason
         reason = '' if reason is None else f': {reason}'
@@ -786,7 +789,7 @@ class ChatControl(EventHelper):
         self.add_info_message(
             self.__format_affiliation_change(
                 event.nick,
-                helpers.get_uf_affiliation(event.affiliation)),
+                get_uf_affiliation(event.affiliation)),
             event.timestamp)
 
     @staticmethod
@@ -814,7 +817,7 @@ class ChatControl(EventHelper):
                                        event: events.MUCUserRoleChanged
                                        ) -> None:
 
-        role = helpers.get_uf_role(event.role)
+        role = get_uf_role(event.role)
         nick = event.nick
 
         reason = event.reason
@@ -837,6 +840,32 @@ class ChatControl(EventHelper):
                                                         reason=reason)
         self.add_info_message(message, event.timestamp)
 
+    def _on_user_hats_changed(self,
+                              _contact: GroupchatContact,
+                              _signal_name: str,
+                              user_contact: GroupchatParticipant,
+                              event: events.MUCUserHatsChanged,
+                              ) -> None:
+
+        self._process_muc_user_hats_changed(event)
+
+    def _process_muc_user_hats_changed(self,
+                                       event: events.MUCUserHatsChanged
+                                       ) -> None:
+
+        if event.hats:
+            hats = ', '.join(event.hats)
+            if event.is_self:
+                message = _('** Your hats are now ') + hats
+            else:
+                message = _('** The hats of %s are now %s') % (event.nick, hats)
+        else:
+            if event.is_self:
+                message = _('** You lost all your hats!')
+            else:
+                message = _('** %s lost all their hats!') % (event.nick,)
+        self.add_info_message(message, event.timestamp)
+
     def _on_user_status_show_changed(self,
                                      contact: GroupchatContact,
                                      _signal_name: str,
@@ -871,7 +900,7 @@ class ChatControl(EventHelper):
         nick = event.nick
         status = event.status
         status = '' if not status else f' - {status}'
-        show = helpers.get_uf_show(event.show_value)
+        show = get_uf_show(event.show_value)
 
         if event.is_self:
             message = _('You are now {show}{status}').format(show=show,
