@@ -101,6 +101,9 @@ class MessageRowActions(Gtk.EventBox):
         self.connect('enter-notify-event', self._on_hover)
         self.connect('leave-notify-event', self._on_hover)
 
+        self.add_events(Gdk.EventMask.SCROLL_MASK)
+        self.connect('scroll-event', self._on_scroll)
+
     def hide_actions(self) -> None:
         # Set a 10ms timeout, which gives us enough time to inhibit hiding
         # if the cursor enters (cursor changes from row to MessageRowActions)
@@ -235,8 +238,20 @@ class MessageRowActions(Gtk.EventBox):
 
         return True
 
+    def _on_scroll(self, _widget: MessageRowActions, event: Gdk.EventScroll) -> bool:
+        # Emulate scroll behavior if cursor rests on MessageRowActions while scrolling
+        if event.direction == Gdk.ScrollDirection.UP:
+            app.window.activate_action('scroll-view-up')
+
+        if event.direction == Gdk.ScrollDirection.DOWN:
+            app.window.activate_action('scroll-view-down')
+
+        return Gdk.EVENT_PROPAGATE
+
     def _on_reply_clicked(self, _button: Gtk.Button) -> None:
-        assert self._message_row is not None
+        if self._message_row is None:
+            return
+
         app.window.activate_action('reply', GLib.Variant('u', self._message_row.pk))
 
     def _on_quick_reaction_button_clicked(self, button: QuickReactionButton) -> None:
@@ -250,11 +265,15 @@ class MessageRowActions(Gtk.EventBox):
         self._send_reaction(emoji, toggle=False)
 
     def _send_reaction(self, emoji: str, toggle: bool = True) -> None:
-        assert self._message_row is not None
+        if self._message_row is None:
+            return
+
         self._message_row.send_reaction(emoji, toggle)
 
     def _on_more_clicked(self, button: Gtk.Button) -> None:
-        assert self._message_row is not None
+        if self._message_row is None:
+            return
+
         self._menu_button_clicked = True
         self._message_row.show_chat_row_menu(self, button)
 
