@@ -244,24 +244,30 @@ class DebugConsoleWindow(Gtk.ApplicationWindow, EventHelper):
 
     def _on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> None:
         if event.keyval == Gdk.KEY_Escape:
-            if self._ui.search_revealer.get_child_revealed():
-                self._ui.search_revealer.set_reveal_child(False)
+            if self._ui.search_toggle.get_active():
+                self._ui.search_toggle.set_active(False)
                 return
+
             self.destroy()
+
         if (event.state & Gdk.ModifierType.CONTROL_MASK and
                 event.keyval == Gdk.KEY_Return or
                 event.keyval == Gdk.KEY_KP_Enter):
             self._on_send()
+
         if (event.state & Gdk.ModifierType.CONTROL_MASK and
                 event.keyval == Gdk.KEY_Up):
             self._on_paste_previous()
+
         if (event.state & Gdk.ModifierType.CONTROL_MASK and
                 event.keyval == Gdk.KEY_Down):
             self._on_paste_next()
+
         if (event.state & Gdk.ModifierType.CONTROL_MASK and
                 event.keyval == Gdk.KEY_f):
             self._ui.search_toggle.set_active(
-                not self._ui.search_revealer.get_child_revealed())
+                not self._ui.search_toggle.get_active())
+
         if event.keyval == Gdk.KEY_F3:
             self._find(Direction.NEXT)
 
@@ -305,8 +311,15 @@ class DebugConsoleWindow(Gtk.ApplicationWindow, EventHelper):
                 # set jabber:client as toplevel namespace
                 # Use type Protocol so nbxmpp counts the stanza for
                 # stream management
-                node = nbxmpp.Protocol(node=stanza,
-                                       attrs={'xmlns': 'jabber:client'})
+                try:
+                    node = nbxmpp.Protocol(
+                        node=stanza,
+                        attrs={'xmlns': 'jabber:client'}
+                    )
+                except Exception as error:
+                    ErrorDialog(_('Invalid Stanza'), str(error))
+                    return
+
             client = app.get_client(self._selected_send_account)
             assert isinstance(node, nbxmpp.Protocol)
             client.connection.send_stanza(node)
