@@ -12,26 +12,21 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
+from typing import TYPE_CHECKING
 
+import hashlib
 import logging
+import platform
 import sys
 import textwrap
+from pathlib import Path
 
+import cairo
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import Gio
@@ -40,6 +35,7 @@ from gi.repository import Gtk
 from nbxmpp.protocol import JID
 
 from gajim.common import app
+from gajim.common import configpaths
 from gajim.common import events
 from gajim.common import ged
 from gajim.common.client import Client
@@ -50,24 +46,43 @@ from gajim.common.ged import EventHelper
 from gajim.common.helpers import allow_showing_notification
 from gajim.common.helpers import play_sound
 from gajim.common.i18n import _
-from gajim.common.modules.contacts import BareContact
-from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import ResourceContact
 
+from gajim.gtk.avatar import merge_avatars
 from gajim.gtk.builder import get_builder
 from gajim.gtk.structs import AccountJidParam
 from gajim.gtk.structs import OpenEventActionParams
 from gajim.gtk.util import add_css_to_widget
-from gajim.gtk.util import get_monitor_scale_factor
 from gajim.gtk.util import get_total_screen_geometry
+from gajim.gtk.util import load_icon_surface
+
+MIN_WINDOWS_TOASTS_WIN_VERSION = 10240
+
+if ((sys.platform == 'win32' and
+        int(platform.version().split('.')[2]) >= MIN_WINDOWS_TOASTS_WIN_VERSION) or
+        TYPE_CHECKING):
+    # Importing windows_toasts on an unsupported Windows version will throw an Exception
+    import winreg
+
+    from windows_toasts import InteractableWindowsToaster
+    from windows_toasts import Toast
+    from windows_toasts import ToastActivatedEventArgs
+    from windows_toasts import ToastButton
+    from windows_toasts import ToastDisplayImage
+    from windows_toasts import ToastImage
+    from windows_toasts import ToastImagePosition
+
+
+WINDOWS_TOAST_NOTIFIER_AUMID = 'Gajim.ToastNotification'
 
 log = logging.getLogger('gajim.gtk.notification')
 
-
 NOTIFICATION_ICONS: dict[str, str] = {
-    'incoming-message': 'gajim-chat_msg_recv',
-    'group-chat-invitation': 'gajim-gc_invitation',
+    'incoming-message': 'gajim-chat-msg-recv',
+    'group-chat-invitation': 'gajim-group-chat-invitation',
     'incoming-call': 'call-start-symbolic',
-    'subscription_request': 'gajim-subscription_request',
+    'subscription_request': 'gajim-subscription-request',
     'unsubscribed': 'gajim-unsubscribed',
     'file-request-received': 'document-send',
     'file-send-error': 'dialog-error',
@@ -141,14 +156,14 @@ class DummyBackend(NotificationBackend):
         pass
 
 
-class Windows(NotificationBackend):
+class WindowsLegacy(NotificationBackend):
     def __init__(self):
         NotificationBackend.__init__(self)
         self._active_notification = None
 
     def _send(self, event: events.Notification) -> None:
         timeout = app.settings.get('notification_timeout')
-        self._withdraw()
+        self._withdraw([])
         self._active_notification = PopupNotification(event, timeout)
 
         def _on_popup_destroy(_widget: Gtk.Window) -> None:
@@ -156,7 +171,7 @@ class Windows(NotificationBackend):
 
         self._active_notification.connect('destroy', _on_popup_destroy)
 
-    def _withdraw(self, *args: Any) -> None:
+    def _withdraw(self, details: list[Any]) -> None:
         if self._active_notification is not None:
             self._active_notification.destroy()
 
@@ -193,8 +208,7 @@ class PopupNotification(Gtk.Window):
 
         if event.type == 'incoming-message':
             assert event.jid is not None
-            pixbuf = _get_avatar_for_notification(
-                event.account, event.jid)
+            pixbuf = _get_pixbuf_icon(event.account, event.jid, event.resource)
             self._ui.image.set_from_pixbuf(pixbuf)
         else:
             icon_name = self._get_icon_name(event)
@@ -221,7 +235,7 @@ class PopupNotification(Gtk.Window):
         if event.icon_name is not None:
             return event.icon_name
         icon_name = event.sub_type or event.type
-        return NOTIFICATION_ICONS.get(icon_name, 'gajim-chat_msg_recv')
+        return NOTIFICATION_ICONS.get(icon_name, 'gajim-chat-msg-recv')
 
     def _add_background_color(self, event: events.Notification) -> None:
         event_type = event.sub_type or event.type
@@ -276,6 +290,147 @@ class PopupNotification(Gtk.Window):
             GLib.source_remove(self._timeout_id)
 
 
+class WindowsToastNotification(NotificationBackend):
+    def __init__(self):
+        NotificationBackend.__init__(self)
+        self._register_notifier_aumid()
+
+        self._toaster = InteractableWindowsToaster(
+            applicationText='Gajim',
+            notifierAUMID=WINDOWS_TOAST_NOTIFIER_AUMID
+        )
+
+    def _register_notifier_aumid(self) -> None:
+        '''Register an AUMID for Gajim's toast notifications.
+        This allows notifications issued by Gajim to have the right icon and title.
+        Code taken from: https://github.com/DatGuy1/Windows-Toasts/blob/main/scripts/register_hkey_aumid.py
+        '''
+        key_path = f'SOFTWARE\\Classes\\AppUserModelId\\{WINDOWS_TOAST_NOTIFIER_AUMID}'
+
+        image_path = (
+            Path(sys.executable).parent.parent
+            / 'share'
+            / 'icons'
+            / 'hicolor'
+            / '96x96'
+            / 'apps'
+            / 'gajim.png'
+        )
+        winreg.ConnectRegistry(None, winreg.HKEY_CURRENT_USER)
+        with winreg.CreateKeyEx(winreg.HKEY_CURRENT_USER, key_path) as master_key:
+            winreg.SetValueEx(master_key, 'DisplayName', 0, winreg.REG_SZ, 'Gajim')
+            winreg.SetValueEx(
+                master_key,
+                'IconUri',
+                0,
+                winreg.REG_SZ,
+                str(image_path.resolve())
+            )
+
+    def _send(self, event: events.Notification) -> None:
+        toast = Toast()
+        toast.text_fields = [
+            event.title,
+            event.text
+        ]
+
+        toast_image = self._get_toast_image(event)
+        toast_display_image = ToastDisplayImage(
+            image=toast_image,
+            position=ToastImagePosition.AppLogo,
+        )
+        toast.AddImage(toast_display_image)
+
+        for button in self._get_toast_buttons(event):
+            toast.AddAction(button)
+
+        toast.on_activated = self._on_activated
+
+        self._toaster.show_toast(toast)
+
+    def _withdraw(self, details: list[Any]) -> None:
+        self._toaster.clear_toasts()
+
+    def _on_activated(self, event: ToastActivatedEventArgs) -> None:
+        # Calls need to be executed with GLib.idle_add to avoid threading issues,
+        # because Toasts run in a different thread.
+        if event.arguments is None:
+            GLib.idle_add(app.window.present_with_time, Gtk.get_current_event_time())
+            return
+
+        if event.arguments.startswith('open-event-'):
+            serialized_data = event.arguments.split('open-event-')[1]
+            params = OpenEventActionParams.from_serialized_string(
+                serialized_data, GLib.VariantType('a{sv}'))
+            if params is not None:
+                GLib.idle_add(
+                    app.app.activate_action,
+                    f'{params.account}-open-event',
+                    params.to_variant()
+                )
+                return
+
+        elif event.arguments.startswith('mark-as-read-'):
+            serialized_data = event.arguments.split('mark-as-read-')[1]
+            params = AccountJidParam.from_serialized_string(
+                serialized_data, GLib.VariantType('a{sv}'))
+            if params is not None:
+                GLib.idle_add(
+                    app.app.activate_action,
+                    f'{params.account}-mark-as-read',
+                    params.to_variant()
+                )
+                return
+
+        GLib.idle_add(app.window.present_with_time, Gtk.get_current_event_time())
+
+    def _get_toast_image(self, event: events.Notification) -> ToastImage:
+        if event.type == 'incoming-message':
+            assert event.jid is not None
+            surface = _get_surface_for_notification(
+                event.account, event.jid, event.resource)
+            return ToastImage(_get_path_for_icon(surface))
+
+        if event.icon_name is not None:
+            surface = load_icon_surface(event.icon_name, 32)
+            assert surface is not None
+            return ToastImage(_get_path_for_icon(surface))
+
+        icon_name = event.sub_type or event.type
+        icon_name = NOTIFICATION_ICONS.get(icon_name, 'mail-unread')
+        surface = load_icon_surface(icon_name, 32)
+        assert surface is not None
+        return ToastImage(_get_path_for_icon(surface))
+
+    def _get_toast_buttons(self, event: events.Notification) -> list[ToastButton]:
+        toast_buttons: list[ToastButton] = []
+
+        jid = '' if event.jid is None else str(event.jid)
+
+        params = OpenEventActionParams(type=event.type,
+                                       sub_type=event.sub_type or '',
+                                       account=event.account,
+                                       jid=jid)
+
+        button = ToastButton(
+            content=_('Open'),
+            arguments=f'open-event-{params.serialize()}'
+        )
+        toast_buttons.append(button)
+
+        if event.type == 'incoming-message':
+            assert isinstance(event.jid, JID)
+            params = AccountJidParam(account=event.account, jid=event.jid)
+
+            button = ToastButton(
+                content=_('Mark as Read'),
+                arguments=f'mark-as-read-{params.serialize()}'
+            )
+            toast_buttons.append(button)
+
+        return toast_buttons
+
+
 class Linux(NotificationBackend):
 
     _action_types = [
@@ -291,20 +446,23 @@ class Linux(NotificationBackend):
 
     def __init__(self):
         NotificationBackend.__init__(self)
-        self._dbus_available: bool = False
+        self._notifications_supported: bool = False
         self._caps: list[str] = []
         self._detect_dbus_caps()
         log.info('Detected notification capabilities: %s', self._caps)
 
     def _detect_dbus_caps(self) -> None:
-        if app.desktop_env == 'gnome':
+        if app.is_flatpak() or app.desktop_env == 'gnome':
             # Gnome Desktop does not use org.freedesktop.Notifications.
             # It has its own API at org.gtk.Notifications, which is not an
             # implementation of the freedesktop spec. There is no documentation
             # on what it currently supports, we can assume at least what the
             # GLib.Notification API offers (icons, actions).
+            #
+            # If the app is run as flatpak the portal API is used
+            # https://flatpak.github.io/xdg-desktop-portal/docs
             self._caps = ['actions']
-            self._dbus_available = True
+            self._notifications_supported = True
             return
 
         def on_proxy_ready(_source: Gio.DBusProxy,
@@ -315,7 +473,7 @@ class Linux(NotificationBackend):
             except GLib.Error as error:
                 log.warning('Notifications D-Bus not available: %s', error)
             else:
-                self._dbus_available = True
+                self._notifications_supported = True
                 log.info('Notifications D-Bus connected')
 
         log.info('Connecting to Notifications D-Bus')
@@ -329,7 +487,7 @@ class Linux(NotificationBackend):
                                   on_proxy_ready)
 
     def _send(self, event: events.Notification) -> None:
-        if not self._dbus_available:
+        if not self._notifications_supported:
             return
 
         notification = Gio.Notification()
@@ -394,9 +552,13 @@ class Linux(NotificationBackend):
         return None
 
     def _make_icon(self, event: events.Notification) -> Gio.Icon:
-        if (event.type == 'incoming-message' and app.desktop_env == 'gnome'):
+        if event.type == 'incoming-message':
             assert event.jid is not None
-            return _get_avatar_for_notification(event.account, event.jid)
+
+            if app.is_flatpak():
+                return _get_bytes_icon(event.account, event.jid, event.resource)
+
+            return _get_file_icon(event.account, event.jid, event.resource)
 
         if event.icon_name is not None:
             return Gio.ThemedIcon.new(event.icon_name)
@@ -406,7 +568,7 @@ class Linux(NotificationBackend):
         return Gio.ThemedIcon.new(icon_name)
 
     def _withdraw(self, details: list[Any]) -> None:
-        if not self._dbus_available:
+        if not self._notifications_supported:
             return
         notification_id = self._make_id(details)
 
@@ -418,26 +580,73 @@ class Linux(NotificationBackend):
         return ','.join(map(str, details))
 
 
-def _get_avatar_for_notification(account: str,
-                                 jid: JID | str) -> GdkPixbuf.Pixbuf:
-    scale = get_monitor_scale_factor()
+def _get_participant_surface(
+        gc_contact: GroupchatContact,
+        resource: str) -> cairo.ImageSurface:
+    size = AvatarSize.NOTIFICATION
+    muc_avatar = gc_contact.get_avatar(size, 1)
+    participant_avatar = gc_contact.get_resource(resource).get_avatar(
+        size, 1, add_show=False)
+
+    return merge_avatars(muc_avatar, participant_avatar)
+
+
+def _get_surface_for_notification(account: str,
+                                  jid: JID | str,
+                                  resource: str | None) -> cairo.ImageSurface:
     size = AvatarSize.NOTIFICATION
     client = app.get_client(account)
     contact = client.get_module('Contacts').get_contact(jid)
-    assert isinstance(contact, BareContact | GroupchatParticipant)
-    surface = contact.get_avatar(size, scale)
+    if isinstance(contact, GroupchatContact):
+        if resource:
+            return _get_participant_surface(contact, resource)
+        else:
+            return contact.get_avatar(size, 1)
+
+    assert not isinstance(contact, ResourceContact)
+    return contact.get_avatar(size, 1, add_show=False)
+
+
+def _get_pixbuf_icon(
+        account: str, jid: JID | str, resource: str | None) -> GdkPixbuf.Pixbuf:
+    size = AvatarSize.NOTIFICATION
+    surface = _get_surface_for_notification(account, jid, resource)
     pixbuf = Gdk.pixbuf_get_from_surface(surface,
                                          0,
                                          0,
-                                         size * scale,
-                                         size * scale)
+                                         size,
+                                         size)
     assert pixbuf is not None
     return pixbuf
 
 
+def _get_bytes_icon(
+        account: str, jid: JID | str, resource: str | None) -> Gio.BytesIcon:
+    pixbuf = _get_pixbuf_icon(account, jid, resource)
+    _, data = pixbuf.save_to_bufferv('png')
+    return Gio.BytesIcon(bytes=GLib.Bytes.new(data))
+
+
+def _get_file_icon(
+        account: str, jid: JID | str, resource: str | None) -> Gio.FileIcon:
+    surface = _get_surface_for_notification(account, jid, resource)
+    path = _get_path_for_icon(surface)
+    return Gio.FileIcon(file=Gio.File.new_for_path(str(path)))
+
+
+def _get_path_for_icon(surface: cairo.ImageSurface) -> Path:
+    path = configpaths.get('AVATAR_ICONS') / hashlib.sha1(
+        bytes(surface.get_data())).hexdigest()
+    if not path.exists():
+        surface.write_to_png(str(path))
+    return path
+
+
 def get_notification_backend() -> NotificationBackend:
     if sys.platform == 'win32':
-        return Windows()
+        if int(platform.version().split('.')[2]) >= MIN_WINDOWS_TOASTS_WIN_VERSION:
+            return WindowsToastNotification()
+        return WindowsLegacy()
 
     if sys.platform == 'darwin':
         return DummyBackend()
