@@ -1,23 +1,11 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
 from typing import cast
-from typing import Optional
-from typing import Union
 
 import locale
 from enum import IntEnum
@@ -42,26 +30,30 @@ from gajim.common.const import AvatarSize
 from gajim.common.const import Direction
 from gajim.common.const import MUC_DISCO_ERRORS
 from gajim.common.const import URIType
-from gajim.common.helpers import get_group_chat_nick
-from gajim.common.helpers import parse_uri
 from gajim.common.helpers import to_user_string
-from gajim.common.helpers import validate_jid
 from gajim.common.i18n import _
 from gajim.common.i18n import get_rfc5646_lang
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.util import as_task
+from gajim.common.util.jid import validate_jid
+from gajim.common.util.muc import get_group_chat_nick
+from gajim.common.util.text import get_country_flag_from_code
+from gajim.common.util.uri import parse_uri
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.chat_filter import ChatFilter
 from gajim.gtk.groupchat_info import GroupChatInfoScrolled
 from gajim.gtk.groupchat_nick import NickChooser
 from gajim.gtk.menus import get_start_chat_row_menu
+from gajim.gtk.tooltips import ContactTooltip
 from gajim.gtk.util import AccountBadge
 from gajim.gtk.util import GajimPopover
-from gajim.gtk.util import get_icon_name
+from gajim.gtk.util import get_status_icon_name
+from gajim.gtk.util import GroupBadge
+from gajim.gtk.util import IdleBadge
 
-ContactT = Union[BareContact, GroupchatContact]
+ContactT = BareContact | GroupchatContact
 
 
 class Search(IntEnum):
@@ -71,8 +63,8 @@ class Search(IntEnum):
 
 class StartChatDialog(Gtk.ApplicationWindow):
     def __init__(self,
-                 initial_jid: Optional[str] = None,
-                 initial_message: Optional[str] = None
+                 initial_jid: str | None = None,
+                 initial_message: str | None = None
                  ) -> None:
 
         Gtk.ApplicationWindow.__init__(self)
@@ -84,7 +76,7 @@ class StartChatDialog(Gtk.ApplicationWindow):
         self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
         self.set_default_size(-1, 600)
         self.ready_to_destroy = False
-        self._parameter_form: Optional[MuclumbusResult] = None
+        self._parameter_form: MuclumbusResult | None = None
         self._keywords: list[str] = []
         self._destroyed = False
         self._search_stopped = False
@@ -97,10 +89,10 @@ class StartChatDialog(Gtk.ApplicationWindow):
         self._ui.join_box.pack_start(self._nick_chooser, True, False, 0)
 
         # Helper for the case where we don't receive a disco info
-        self._new_chat_row: Optional[ContactRow] = None
+        self._new_chat_row: ContactRow | None = None
         self._search_is_valid_jid = False
 
-        self.new_contact_rows: dict[str, Optional[ContactRow]] = {}
+        self.new_contact_rows: dict[str, ContactRow | None] = {}
         self._accounts = app.get_enabled_accounts_with_labels()
 
         rows: list[ContactRow] = []
@@ -144,7 +136,7 @@ class StartChatDialog(Gtk.ApplicationWindow):
         if rows:
             self._load_contacts(rows)
 
-        self._initial_message: dict[str, Optional[str]] = {}
+        self._initial_message: dict[str, str | None] = {}
         if initial_jid is not None:
             self._initial_message[initial_jid] = initial_message
             self._ui.search_entry.set_text(initial_jid)
@@ -305,7 +297,8 @@ class StartChatDialog(Gtk.ApplicationWindow):
                 return Gdk.EVENT_STOP
 
             if self._current_listbox_is(Search.GLOBAL):
-                if self._ui.search_entry.is_focus():
+                if (self._ui.search_entry.is_focus() and
+                        self._ui.search_entry.get_text()):
                     self._global_search_listbox.remove_all()
                     self._start_search()
 
@@ -339,6 +332,9 @@ class StartChatDialog(Gtk.ApplicationWindow):
         self._ui.listbox.invalidate_filter()
 
     def _start_new_chat(self, row: ContactRow) -> None:
+        if row.jid is None:
+            return
+
         if row.is_new:
             try:
                 validate_jid(row.jid)
@@ -349,7 +345,6 @@ class StartChatDialog(Gtk.ApplicationWindow):
             self._disco_info(row)
             return
 
-        jid = JID.from_string(row.jid)
         if row.groupchat:
             if not app.account_is_available(row.account):
                 self._show_error_page(_('You can not join a group chat '
@@ -357,20 +352,20 @@ class StartChatDialog(Gtk.ApplicationWindow):
                 return
 
             self.ready_to_destroy = True
-            if app.window.chat_exists(row.account, jid):
-                app.window.select_chat(row.account, jid)
+            if app.window.chat_exists(row.account, row.jid):
+                app.window.select_chat(row.account, row.jid)
                 self.destroy()
                 return
 
             self.ready_to_destroy = False
             self._redirected = False
-            self._disco_muc(row.account, jid, request_vcard=row.is_new)
+            self._disco_muc(row.account, row.jid, request_vcard=row.is_new)
 
         else:
-            initial_message = self._initial_message.get(str(jid))
+            initial_message = self._initial_message.get(str(row.jid))
             app.window.add_chat(
                 row.account,
-                jid,
+                row.jid,
                 'contact',
                 select=True,
                 message=initial_message)
@@ -416,7 +411,10 @@ class StartChatDialog(Gtk.ApplicationWindow):
             self._show_error_page(_('This address is not reachable.'))
             return
 
-        if result.is_muc:
+        if result.is_muc and not result.jid.is_domain:
+            # This is mostly a fix for the MUC protocol, there is no
+            # way to differentiate between a MUC service and room.
+            # Except the MUC XEP defines rooms should have a localpart.
             row.update_chat_type(groupchat=True)
         else:
             row.update_chat_type()
@@ -457,7 +455,7 @@ class StartChatDialog(Gtk.ApplicationWindow):
         else:
             self._set_error_from_code('not-muc-service')
 
-    def _set_error(self, error: Union[StanzaError, TimeoutStanzaError]) -> None:
+    def _set_error(self, error: StanzaError | TimeoutStanzaError) -> None:
         if isinstance(error, TimeoutStanzaError):
             text = _('This address is not reachable.')
         else:
@@ -556,7 +554,7 @@ class StartChatDialog(Gtk.ApplicationWindow):
             search_entry.set_text(uri.data['jid'])
             return
 
-        if '@' in search_text:
+        if search_text:
             try:
                 validate_jid(search_text)
             except ValueError:
@@ -734,7 +732,7 @@ class StartChatDialog(Gtk.ApplicationWindow):
                                parameters: bool = False
                                ) -> None:
         if self._search_stopped:
-            raise CancelledError()
+            raise CancelledError
 
         if is_error(result):
             assert isinstance(result, StanzaError)
@@ -759,9 +757,9 @@ class StartChatDialog(Gtk.ApplicationWindow):
 class ContactRow(Gtk.ListBoxRow):
     def __init__(self,
                  account: str,
-                 contact: Optional[ContactT],
-                 jid: Optional[JID],
-                 name: Optional[str],
+                 contact: ContactT | None,
+                 jid: JID | None,
+                 name: str | None,
                  show_account: bool,
                  groupchat: bool = False
                  ) -> None:
@@ -784,38 +782,60 @@ class ContactRow(Gtk.ListBoxRow):
         image.set_size_request(AvatarSize.CHAT, AvatarSize.CHAT)
         grid.add(image)
 
-        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
-        box.set_hexpand(True)
+        self._tooltip = ContactTooltip()
+        image.set_has_tooltip(True)
+        image.connect('query-tooltip', self._on_query_tooltip)
 
         if self.name is None:
             self.name = _('Start New Chat')
 
+        meta_box = Gtk.Box(
+            orientation=Gtk.Orientation.VERTICAL, valign=Gtk.Align.CENTER)
+
         self.name_label = Gtk.Label(label=self.name)
         self.name_label.set_ellipsize(Pango.EllipsizeMode.END)
         self.name_label.set_xalign(0)
         self.name_label.set_width_chars(20)
         self.name_label.set_halign(Gtk.Align.START)
-        self.name_label.get_style_context().add_class('bold16')
-        name_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=6)
-        name_box.add(self.name_label)
+        self.name_label.get_style_context().add_class('bold14')
+        meta_box.add(self.name_label)
+
+        if contact and not contact.is_groupchat:
+            if idle := contact.idle_datetime:
+                idle_badge = IdleBadge(idle)
+                meta_box.add(idle_badge)
+
+        if contact and not contact.is_groupchat and (status := contact.status):
+            self.status_label = Gtk.Label(
+                label=status,
+                ellipsize=Pango.EllipsizeMode.END,
+                xalign=0,
+                width_chars=22,
+                halign=Gtk.Align.START,
+            )
+            self.status_label.get_style_context().add_class('dim-label')
+            self.status_label.get_style_context().add_class('small-label')
+            meta_box.add(self.status_label)
+
+        grid.attach(meta_box, 1, 0, 1, 3)
+
+        badge_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
         if show_account:
             account_badge = AccountBadge(account)
             account_badge.set_halign(Gtk.Align.END)
             account_badge.set_valign(Gtk.Align.START)
             account_badge.set_hexpand(True)
-            name_box.add(account_badge)
-        box.add(name_box)
+            badge_box.add(account_badge)
 
-        self.jid_label = Gtk.Label(label=str(jid))
-        self.jid_label.set_tooltip_text(str(jid))
-        self.jid_label.set_ellipsize(Pango.EllipsizeMode.END)
-        self.jid_label.set_xalign(0)
-        self.jid_label.set_width_chars(22)
-        self.jid_label.set_halign(Gtk.Align.START)
-        self.jid_label.get_style_context().add_class('dim-label')
-        box.add(self.jid_label)
+        if contact and not contact.is_groupchat and not contact.is_pm_contact:
+            groups = contact.groups
+            for group in groups:
+                group_badge = GroupBadge(group)
+                badge_box.add(group_badge)
 
-        grid.add(box)
+        grid.attach(badge_box, 2, 0, 1, 3)
+
+        self._grid = grid
 
         eventbox = Gtk.EventBox()
         eventbox.connect('button-press-event', self._popup_menu)
@@ -823,6 +843,18 @@ class ContactRow(Gtk.ListBoxRow):
         self.add(eventbox)
         self.show_all()
 
+    def _on_query_tooltip(self,
+                          _img: Gtk.Image,
+                          _x_coord: int,
+                          _y_coord: int,
+                          _keyboard_mode: bool,
+                          tooltip: Gtk.Tooltip) -> bool:
+        if not isinstance(self.contact, BareContact):
+            return False
+        v, widget = self._tooltip.get_tooltip(self.contact)
+        tooltip.set_custom(widget)
+        return v
+
     def _popup_menu(self,
                     _widget: Gtk.EventBox,
                     event: Gdk.EventButton
@@ -830,7 +862,7 @@ class ContactRow(Gtk.ListBoxRow):
         if not self.groupchat:
             return
 
-        if event.button != 3:  # right click
+        if event.button != Gdk.BUTTON_SECONDARY:
             return
 
         menu = get_start_chat_row_menu(self.account, self.jid)
@@ -842,7 +874,7 @@ class ContactRow(Gtk.ListBoxRow):
         if self.is_new:
             icon_name = 'avatar-default'
             if self.groupchat:
-                icon_name = get_icon_name('muc-inactive')
+                icon_name = get_status_icon_name('muc-inactive')
             return Gtk.Image.new_from_icon_name(icon_name, Gtk.IconSize.DND)
 
         scale = self.get_scale_factor()
@@ -852,7 +884,7 @@ class ContactRow(Gtk.ListBoxRow):
 
     def update_jid(self, jid: JID) -> None:
         self.jid = jid
-        self.jid_label.set_text(str(jid))
+        self._grid.set_tooltip_text(str(jid))
 
     def update_chat_type(self, groupchat: bool = False) -> None:
         self.is_new = False
@@ -861,8 +893,7 @@ class ContactRow(Gtk.ListBoxRow):
     def get_search_text(self) -> str:
         if self.contact is None and not self.groupchat:
             return str(self.jid)
-        if self.show_account:
-            return f'{self.name} {self.jid} {self.account_label}'
+
         return f'{self.name} {self.jid}'
 
 
@@ -871,7 +902,7 @@ class GlobalSearch(Gtk.ListBox):
         Gtk.ListBox.__init__(self)
         self.set_has_tooltip(True)
         self.set_activate_on_single_click(False)
-        self._progress: Optional[ProgressRow] = None
+        self._progress: ProgressRow | None = None
         self._add_placeholder()
         self.show_all()
 
@@ -947,27 +978,77 @@ class GlobalSearch(Gtk.ListBox):
 
 class ResultRow(Gtk.ListBoxRow):
     def __init__(self, item: MuclumbusItem) -> None:
-        Gtk.ListBoxRow.__init__(self)
-        self.set_activatable(True)
+        Gtk.ListBoxRow.__init__(self, activatable=True)
         self.get_style_context().add_class('start-chat-row')
+
         self.is_new = False
         self.jid = JID.from_string(item.jid)
         self.groupchat = True
 
-        name_label = Gtk.Label(label=item.name)
-        name_label.set_halign(Gtk.Align.START)
-        name_label.set_ellipsize(Pango.EllipsizeMode.END)
-        name_label.set_max_width_chars(40)
+        name_label = Gtk.Label(
+            label=item.name,
+            halign=Gtk.Align.START,
+            ellipsize=Pango.EllipsizeMode.END,
+            max_width_chars=40
+        )
         name_label.get_style_context().add_class('bold16')
-        jid_label = Gtk.Label(label=item.jid)
-        jid_label.set_halign(Gtk.Align.START)
-        jid_label.set_ellipsize(Pango.EllipsizeMode.END)
-        jid_label.set_max_width_chars(40)
-        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        box.add(name_label)
-        box.add(jid_label)
 
-        self.add(box)
+        description_label = Gtk.Label(
+            label=item.description,
+            halign=Gtk.Align.START,
+            ellipsize=Pango.EllipsizeMode.END,
+            max_width_chars=40
+        )
+        description_label.get_style_context().add_class('dim-label')
+
+        main_box = Gtk.Box(spacing=12)
+        self.add(main_box)
+
+        name_box = Gtk.Box(
+            orientation=Gtk.Orientation.VERTICAL,
+            hexpand=True,
+            tooltip_text=item.jid
+        )
+        name_box.add(name_label)
+        name_box.add(description_label)
+
+        main_box.add(name_box)
+
+        language_code = item.language.upper()
+        if len(language_code) < 2:
+            language_code = ''
+
+        if len(language_code) > 2:
+            # Fix if no ISO code has been provided
+            language_code = language_code[:2]
+
+        if language_code == 'EN':
+            # Fix for most used languages/countries
+            language_code = 'GB'
+
+        language_label = Gtk.Label(
+            label=get_country_flag_from_code(language_code),
+            tooltip_text=_('Language: %s') % item.language
+        )
+        main_box.add(language_label)
+
+        users_box = Gtk.Box(
+            spacing=6,
+            tooltip_text=_('Participants Count')
+        )
+        users_box.get_style_context().add_class('dim-label')
+
+        users_icon = Gtk.Image.new_from_icon_name(
+            'system-users-symbolic',
+            Gtk.IconSize.BUTTON
+        )
+        users_box.add(users_icon)
+
+        users_count_label = Gtk.Label(label=item.nusers)
+        users_box.add(users_count_label)
+
+        main_box.add(users_box)
+
         self.show_all()
 
 
