@@ -39,17 +39,17 @@ from gajim.common.const import AvatarSize
 from gajim.common.const import Display
 from gajim.common.const import LOCATION_DATA
 from gajim.common.ged import EventHelper as CommonEventHelper
-from gajim.common.helpers import format_idle_time
-from gajim.common.helpers import URL_REGEX
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.regex import URL_REGEX
 from gajim.common.storage.archive import models as mod
 from gajim.common.storage.archive.const import ChatDirection
 from gajim.common.storage.archive.const import MessageType
 from gajim.common.structs import VariantMixin
 from gajim.common.styling import PlainBlock
+from gajim.common.util.user_strings import format_idle_time
 
 from gajim.gtk.const import WINDOW_MODULES
 
@@ -146,22 +146,11 @@ def load_icon_pixbuf(icon_name: str,
     return icon
 
 
-def get_app_icon_list(scale_widget: Gtk.Widget) -> list[GdkPixbuf.Pixbuf]:
-    scale = scale_widget.get_scale_factor()
-    pixbufs: list[GdkPixbuf.Pixbuf] = []
-    for size in (16, 32, 48, 64, 128):
-        pixbuf = load_icon_pixbuf('org.gajim.Gajim', size=size, scale=scale)
-        if pixbuf is not None:
-            pixbufs.append(pixbuf)
-    return pixbufs
-
-
-def get_icon_name(name: str,
-                  transport: str | None = None) -> str:
-    if transport is not None:
-        return f'{transport}-{name}'
-
-    return f'dcraven-{name}'
+def get_status_icon_name(name: str) -> str:
+    prefix = 'gajim'
+    if app.is_flatpak():
+        prefix = app.get_default_app_id()
+    return f'{prefix}-status-{name}'
 
 
 def load_user_iconsets() -> None:
@@ -980,14 +969,16 @@ class GdkRectangle(Gdk.Rectangle):
 
 class GajimPopover(Gtk.Popover):
     def __init__(self,
-                 menu: Gio.MenuModel,
+                 menu: Gio.MenuModel | None = None,
                  relative_to: Gtk.Widget | None = None,
                  position: Gtk.PositionType = Gtk.PositionType.RIGHT,
                  event: Gdk.EventButton | None = None) -> None:
 
         Gtk.Popover.__init__(self)
 
-        self.bind_model(menu)
+        if menu is not None:
+            self.bind_model(menu)
+
         self.set_relative_to(relative_to)
         self.set_position(position)
         if event is not None:
